﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Emgu.CV.Structure;
using Emgu.CV;
using Emgu.CV.UI;
using System.Drawing;
using System.Runtime.InteropServices;
using Emgu.CV.Util;


namespace TP
{
    class ParamDetecterGraine
    {
        private Image<Bgr, Byte> imageNir;
        Rectangle ROICellule;
        List<Recipient> listeContenant;
        int noCellule;
        typeGraine typeG = new typeGraine();
		double Fcorr; // facteur de rehaussement radiometrique

        /* ----------------------------------------------------------------------------------------------------------
         *  Initialisation de la classe 
         *  ---------------------------------------------------------------------------------------------------------*/


         public ParamDetecterGraine(Image<Bgr, Byte> image, Rectangle roi, List<Recipient> liste, int no, typeGraine t)
         {
             imageNir = image;
             ROICellule = roi;
             listeContenant = liste;
             noCellule = no;
             typeG = t;
         }
		 
		 public ParamDetecterGraine(Image<Bgr, Byte> image, Rectangle roi,  List<Recipient> liste, int no, typeGraine t, double FcorrRadio)
         {
             imageNir = image;
             ROICellule = roi;
             listeContenant = liste;
             noCellule = no;
             typeG = t;
			 Fcorr = FcorrRadio;
         }

    
        /* Detection des graines de type t  dans une cellule no de la liste de cellules */


        public void DetecterGrainesCellule()
        {
			correctionRadiometriqueImageNIR();
            //listeContenant[noCellule].Moyenne = DetecterMoyenne();
			listeContenant[noCellule].Moyenne[1] = EstimerMoyenneSpheigne();
            listeContenant[noCellule].ListeGraine = DetecterGraine(listeContenant[noCellule].Moyenne[1]); /*passe vecteur de moyenne*/
				  
        }
		
		public double TrouverSeuilOptimalCassette(Image<Gray, Byte> im1, int tAmax, int tAmin)
         {

             try
             {
                 int[] hist = new int[256];

                 for (int k = 0; k < 256; k++)
                 {
                     hist[k] = 0;
                 }

                 for (int i = im1.ROI.Y+10; i < im1.ROI.Y+im1.ROI.Height-10; i++)
                 {
                     for (int j = im1.ROI.X+10; j < im1.ROI.X+im1.ROI.Width-10; j++)
                     {
                         int val = im1.Data[i, j, 0];
                         hist[val]++;
                     }
                 }

                 double sumT = 0.0;

                 for (int t = 0; t < 256; t++)
                     sumT += (double)(t) * (double)(hist[t]);

                 double sumB = 0.0;
                 int wB = 0;
                 int wF = 0;

                 double varmax = 0.0;
                 int tOptimal = 0;
				 
				 for (int t = 0; t < tAmax; t++)
                 {
                     wB += hist[t];
				 }

                 for (int t = tAmax; t < tAmin; t++)
                 {
                     wB += hist[t];
                     if (wB == 0)
                         continue;

                     wF = (im1.ROI.Width-20) * (im1.ROI.Height-20) - wB;
                     if (wF == 0)
                         break;

                     sumB += (double)(t) * (double)(hist[t]);

                     double mB = sumB / (double)(wB);
                     double mF = (sumT - sumB) / (double)(wF);

                     double varbetween = (double)(wB) * (double)(wF) * (mB - mF) * (mB - mF);

                     if (varbetween > varmax)
                     {
                         varmax = varbetween;
                         tOptimal = t;
                     }
                 }


                 return tOptimal;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le seuil optimal de Otsu");
                 return 0;
             }
         }
		
		/* ----------------------------------------------------------------------------------------------------------
         *  Fonction qui permet de corriger radiometriquement l'image imageNir
         *  ---------------------------------------------------------------------------------------------------------*/
         public void correctionRadiometriqueImageNIR()
         {
            double valeur;

             for (int i = imageNir.ROI.Y+10; i < imageNir.ROI.Y+imageNir.ROI.Height-10; i++)
			 {
                 for (int j = imageNir.ROI.X+10; j < imageNir.ROI.X+imageNir.ROI.Width-10; j++)
                 {
					 valeur = Fcorr * imageNir.Data[i, j, 0]);
					 if(valeur > 255)
						 valeur = 255.0;
                     imageNir.Data[i, j, 0] = (byte)valeur;
					 imageNir.Data[i, j, 1] = (byte)valeur;
					 imageNir.Data[i, j, 2] = (byte)valeur;
                         
                 }
             }
            
         }
		 
   /* ----------------------------------------------------------------------------------------------------------
   *  Fonction calculant la variance des pixels de spheigne considerant une bordure de 10 en x et y du contour de platic enleve
   *  ---------------------------------------------------------------------------------------------------------*/
         public double EstimerStdSpheigne(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double var;
             double somme = 0;
             int nb = 0;

             for (int i = imageNir.ROI.Y+10; i < imageNir.ROI.Y+imageNir.ROI.Height-10; i++)
			 {
                 for (int j = imageNir.ROI.X+10; j < imageNir.ROI.X+imageNir.ROI.Width-10; j++)
                 {
                         nb++;
                         somme = somme + ((imageRGBgray.Data[i, j, 0] - moyenne) * (imageRGBgray.Data[i, j, 0] - moyenne));
                 }
             }
             var = somme / (double)nb;

             return Math.Sqrt(var);
         }

		
		 /* ----------------------------------------------------------------------------------------------------------
         *  Fonction calculant la moyenne des pixels de spheigne, bordure de 10 en x et y du contour de platic enleve 
         *  ---------------------------------------------------------------------------------------------------------*/
         public double EstimerMoyenneSpheigne()
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double moy;
             double somme = 0;
             int nb = 0;

             for (int i = imageNir.ROI.Y+10; i < imageNir.ROI.Y+imageNir.ROI.Height-10; i++)
			 {
                 for (int j = imageNir.ROI.X+10; j < imageNir.ROI.X+imageNir.ROI.Width-10; j++)
                 {
                         somme = somme + imageRGBgray.Data[i, j, 0];
                         nb++;
                 }
             }
             moy = somme / (double)nb;

             return moy;
         }
		 
		 public double EstimerMoyenneAmin1Graine(double moySpheigne)
         {

             try
             {
				 int amin = typeG.AireMin;
				 double moySeuilAmin1Graine;
	
				 
				 Image<Gray, Byte> im1 = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
				 
                 double[] hist = new double[256]; // histo de la spheigne
				 
                 for (int k = int (moySpheigne); k < 256; k++)
                 {
                     hist[k]  = 0.0;
                 }

				for (int i = imageNir.ROI.Y+10; i < imageNir.ROI.Y+imageNir.ROI.Height-10; i++)
				{
					for (int j = imageNir.ROI.X+10; j < imageNir.ROI.X+imageNir.ROI.Width-10; j++)
					{
                         int val = im1.Data[i, j, 0];
						 if(val > int (moySpheigne))
							hist[val]++;
                    }
                 }

				int a=0; // aire sous la courbe de histo de 255 a t ou t correspond au nombre de pixel de graines AireMax
				int t=255; 
                 while (a < amin)
				 {
                     a+=hist[t--];
				 }
				// t++;
				 
				 // Calcul de la moyenne des amax pixels de la region de spheigne entre t et 255
				 moySeuilAmin1Graine = 0;
				 for (int j = t; j < 256; j++)
                 {
							moySeuilAmin1Graine += j*hist[j];
                 }
				 moySeuilAmin1Graine /= a;
				 

                 return moySeuilAmin1Graine;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le coefficient de variation");
                 return 0;
             }
         }

		 public double EstimerSeuilAminmax1Graine(double moySpheigne, int aminmax)
         {

             try
             {
				 
				 Image<Gray, Byte> im1 = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
				 
                 double[] hist = new double[256]; // histo de la spheigne
				 
                 for (int k = int (moySpheigne); k < 256; k++)
                 {
                     hist[k]  = 0.0;
                 }

				for (int i = imageNir.ROI.Y+10; i < imageNir.ROI.Y+imageNir.ROI.Height-10; i++)
				{
					for (int j = imageNir.ROI.X+10; j < imageNir.ROI.X+imageNir.ROI.Width-10; j++)
					{
                         int val = im1.Data[i, j, 0];
						 if(val > int (moySpheigne))
							hist[val]++;
                    }
                 }

				int a=0; // aire sous la courbe de histo de 255 a t ou t correspond au nombre de pixel de graines AireMax
				int t=255; 
                 while (a < aminmax)
				 {
                     a+=hist[t--];
				 }
				// t++;
				 

                 return t;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le seuil Aminmax");
                 return 0;
             }
         }

        


 
  		 /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la graine
          *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraine(double moyenneG) /*utilisation de la moyenne pour tresholder*/
         { 
            List<Graine> ListeGraine = new List<Graine>();
            
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
   
		 
		  double moyenneAmin = EstimerMoyenneAmin1Graine(moyenneG);
		  if ((moyenneAmin/moyenneG)-1.0 < 0.5) // moyenne Amin plus grande que 50 %  => presence de graines
			  return ListeGraine;
		  
           // double stdSpheigne = EstimerStdSpheigne(moyenneG);
            
		  tmin = EstimerSeuilAminmax1Graine(moyenneG, typeG.AireMin);
		  tmax = EstimerSeuilAminmax1Graine(moyenneG, typeG.AireMax);
          
          double seuil = TrouverSeuilOptimalCassette(Image<Gray, Byte> im1, int tAmax, int tAmin);
          imageRGBgray._ThresholdBinary(new Gray(seuil), new Gray(255)); 
		   
		   // double seuil = EstimerSeuilAmax1Graine();
			//imageRGBgray._ThresholdBinary(new Gray(seuil), new Gray(255));
			
			//if(moyenneSpheigne +  2*stdSpheigne > 255)
			//	imageRGBgray._ThresholdBinary(new Gray(moyenneSpheigne +  1.5*stdSpheigne), new Gray(255));
			//else
            //imageRGBgray._ThresholdBinary(new Gray(moyenneSpheigne +  2*stdSpheigne), new Gray(255));


             // elemine les spikes
            imageRGBgray.Erode(1).Dilate(1);

            // emplir les trous
           //  imageRGBgray.Dilate(1).Erode(1);

             imageRGBgray.SmoothMedian(7);

             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             
             /*pour chaque contour dans une cellulle*/
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
               //  if (contours.Total >= 6 && contours.Area > ((float)typeG.largeurMin * (float)typeG.hauteurMin * Math.PI) && contours.BoundingRectangle.Width > (float)typeG.largeurMin && contours.BoundingRectangle.Width < (float)typeG.largeurMax &&
                //          contours.BoundingRectangle.Height > (float)typeG.hauteurMin && contours.BoundingRectangle.Height < (float)typeG.hauteurMax)
                // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && contours.Area < (3.0*typeG.aireMax))
                // {

            

                   if (contours.Total >= 6 && /*contours.Perimeter > typeG.pmin && contours.Perimeter < typeG.pmax &&*/ contours.Area > typeG.aireMin && contours.Area < typeG.aireMin)
                   {


                     bool ok = true;
                    /* if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;

                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - typeG.largeurMax / 2))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) >  (imageRGBgray.Height / 2 + typeG.largeurMax / 2))
                         ok = false;
                    * 
                    */ 
            
                    /* if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - 20))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + 20))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - 20))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageRGBgray.Height / 2 + 20))
                         ok = false;*/

                    // if ((contours.Area * 4.0 * Math.PI) / (contours.Perimeter * contours.Perimeter) > .95) // proche d un cercle ?????????
                   //      ok = false;

               //      if ((contours.GetMinAreaRect().size.Height / contours.GetMinAreaRect().size.Width) > 2.0)
                //         ok = false;






 
                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok )
                     {
                        
                        Graine gr = new Graine(contours, 1);
                        
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {

               
                  
                 */
                            /*Calcule le nombre de graines à partir des angles */
                            //  System.Console.WriteLine(angle.Count);
                          //  gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                        /*    Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                            MCvBox2D minR = contours.GetMinAreaRect();

                            if (typeG.nom == "Pin blanc")
                            {
                                gr.Nb_graine = ((contours.Area  > 188) && (minR.size.Height > 43) && (minR.size.Width > 28)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                            }*/


                           
                            //if((contours.Area/cHull.Area) < 0.9) // solidity 
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > typeG.aireMax)) ? gr.Nb_graine + 1 : gr.Nb_graine; // 1 graine ou plus que 2

                            //gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.95) && !((contours.Area > typeG.aireMin) && (contours.Area < typeG.aireMax))) ? gr.Nb_graine + 1 : gr.Nb_graine;
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.90) && (contours.Area > 2.0* typeG.aireMin)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                         ListeGraine.Add(gr);
                        }
                   }
                // }
             
             }
             return ListeGraine;
         }

 /* ----------------------------------------------------------------------------------------------------------
 *  Fonction detectant la graine
 *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraineAvantMai2018(double moyenneG) /*utilisation de la moyenne pour tresholder*/
         { 
            List<Graine> ListeGraine = new List<Graine>();
            
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
           //double std = DetecterStd(moyenneG);
           

           if(moyenneG+2.0*std > 255)
              imageRGBgray._ThresholdBinary(new Gray(moyenneG + 1.5 * std), new Gray(255));
           else
              imageRGBgray._ThresholdBinary(new Gray(moyenneG + 2.0 * std), new Gray(255)); 

             // elemine les spikes
            imageRGBgray.Erode(1).Dilate(1);

            // emplir les trous
           //  imageRGBgray.Dilate(1).Erode(1);

             imageRGBgray.SmoothMedian(7);

             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             
             /*pour chaque contour dans une cellulle*/
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
               //  if (contours.Total >= 6 && contours.Area > ((float)typeG.largeurMin * (float)typeG.hauteurMin * Math.PI) && contours.BoundingRectangle.Width > (float)typeG.largeurMin && contours.BoundingRectangle.Width < (float)typeG.largeurMax &&
                //          contours.BoundingRectangle.Height > (float)typeG.hauteurMin && contours.BoundingRectangle.Height < (float)typeG.hauteurMax)
                // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && contours.Area < (3.0*typeG.aireMax))
                // {

                   // MCvBox2D minR = contours.GetMinAreaRect();

                   // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && minR.size.Height > typeG.hauteurMin && minR.size.Width > typeG.largeurMin)
                   //{

                   if (contours.Total >= 6 && /*contours.Perimeter > typeG.pmin && */ contours.Area > (typeG.aireMax+typeG.aireMin)/2)
                   {
                     bool ok = true;
                   /*  if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;

                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - typeG.largeurMax / 2))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) >  (imageRGBgray.Height / 2 + typeG.largeurMax / 2))
                         ok = false;
                    * 
                    */ 
            
            // Elimination du cadre
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - 60))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + 60))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - 60))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageRGBgray.Height / 2 + 60))
                         ok = false;

            // Elimination des objets longilignes comme des brindilles
                     //if ((contours.GetMinAreaRect().size.Height / contours.GetMinAreaRect().size.Width) > 2.0)
                     //    ok = false;

            // Elimination des objets circulaires associés a la vermiculite

                     //if (typeG.nom == "Épinette blanche")
                     //{
                         if ((contours.Area * 4.0 * Math.PI) / (contours.Perimeter * contours.Perimeter) > .95) // proche d un cercle ?????????
                             ok = false;
                     //}
   
 
                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok )
                     {
                        
                        Graine gr = new Graine(contours, 1);
                        
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {

                        /*http://www.emgu.com/wiki/files/2.0.1.0/html/492603a1-20de-fa97-b128-bd8d55a29e48.htm*/
                /*         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                            MCvConvexityDefect[] defacts = te.ToArray();
                            List<double> angle = new List<double>();
                            foreach (MCvConvexityDefect d in defacts)
                            {
                                double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                                double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                                double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                                double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)
                                
                                if (Math.Abs(fAngle) < 0.6) // > 45 < 135 degres
                                    angle.Add(fAngle);
                          
                               // if (Math.Abs(fAngle) < (float)typeG.angle)
                                //    angle.Add(fAngle);

                                //if (Math.Acos(fAngle) < (float)typeG.angle)
                                 //   angle.Add(fAngle);
                               // if (Math.Acos(fAngle) < 135.0)
                                 //   angle.Add(fAngle);

                            }
                  
                 */
                            /*Calcule le nombre de graines à partir des angles */
                            //  System.Console.WriteLine(angle.Count);
                          //  gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                            Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                            gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > (typeG.aireMin * 1.5))) ? gr.Nb_graine + 1 : gr.Nb_graine; 
                           
                            //if((contours.Area/cHull.Area) < 0.9) // solidity 
                          // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > typeG.aireMax)) ? gr.Nb_graine + 1 : gr.Nb_graine; // 1 graine ou plus que 2

                            //gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.95) && !((contours.Area > typeG.aireMin) && (contours.Area < typeG.aireMax))) ? gr.Nb_graine + 1 : gr.Nb_graine;
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.90) && (contours.Area > 2.0* typeG.aireMin)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                         ListeGraine.Add(gr);
                        }
                   }
                // }
             
             }
             return ListeGraine;
         }

    }
}
